import pandas as pd

def excel_writer(daf, words, filepath) -> None:
    '''Write dataframe into pandas dataframe or excel file'''
    web, name, info = words[1], words[0], words[3]
    if filepath != "":
        filepath += "/"
    writer = pd.ExcelWriter(filepath+web+'_'+name+'_'+info+'.xlsx')
    daf.to_excel(writer)
    writer.close()
    print(f'{name} {info} data is written successfully to Excel File.')
    
def df_writer(daf) -> pd.DataFrame:
    return daf


import matplotlib.pyplot as plt

import mcfinance.extractors
import mcfinance.writer
import mcfinance.retrieval

class Extractor:

    def __init__(self, user_input, years = 10, docs = ["balance sheet","profit loss","cash flow"], filepath = "") -> None:
        '''Initialize extractor with company name, number of years and required documents'''
        self.company = mcfinance.retrieval.comp_name(user_input)
        self.company = self.company.replace(" ","")
        self.years = years
        self.docs = [doc.replace(" ","") for doc in docs]
        self.filepath = filepath
    
    #setter method
    def set_inputs(self, user_input = None, years = None, docs = None, filepath = None) -> None:
        '''Setter method for changing/adding fields'''
        if not(user_input is None):
            self.company = mcfinance.retrieval.comp_name(user_input)
        if not(years is None):
            self.years = years
        if not(docs is None):
            self.docs = docs
        if not(filepath is None):
            self.filepath = docs
        return

    def get_inputs(self)-> None:
        '''Getter method to display stored info'''
        return (self.company, self.years, self.docs, self.filepath)

    

    def get_info(self, option = 1):
        '''option 0: return a dataframe; option 1: write contents to excel file'''
        strg = []
        for doc in self.docs:
        #generate data from docs
            stx = self.company + " moneycontrol consolidated " + doc
            if(option == 1):
                mcfinance.writer.excel_writer(mcfinance.extractors.search_gen(stx, self.years), stx.split(), filepath=self.filepath)
            else:
                df1 = mcfinance.writer.df_writer(mcfinance.extractors.search_gen(search_term=stx, period=self.years))
                strg.append(df1)
        if(option!=1):
            return strg
        else:
            return
    #just works dont TOUCH
    def plotter(self,*attributes):
        '''(experimental) Plots a specific company attribute over selected years'''
        attr = list(attributes)
        adict = dict(zip(attr, self.docs))
        for at, doc in adict.items():
            psearch = self.company + " moneycontrol consolidated " + doc
            X, Y = mcfinance.extractors.plo(attribute= at, search_term= psearch, period= self.years)
            plt.plot(X,Y,"-o", label = self.company)
            plt.xlabel("month-year")
            plt.ylabel(at)
            
            #plt.show()

    @staticmethod
    def cmp_plot(comp, attributes):
        for company in comp:
            company.plotter(attributes)
        plt.legend(bbox_to_anchor=(1.05, 1.0), loc='upper left')
        plt.tight_layout()
        plt.show()




from cachetools import cached, TTLCache
from bs4 import BeautifulSoup
import requests
import time
import pandas as pd
import json
import importlib.resources

report_cache = TTLCache(maxsize=100, ttl=3600) 
url_cache = TTLCache(maxsize=100, ttl=3600) 

@cached(cache=url_cache)
def urlfinder(search_term):
    '''Find google search results'''
    results = 5

    page = requests.get(f"http://www.google.com/search?q={search_term}&num={results}")
    while(page.status_code == 429):
        time.sleep(5)
        page = requests.get(f"http://www.google.com/search?q={search_term}&num={results}")
        if(page.status_code ==200):
            break

    soup1 = BeautifulSoup(page.content, "html5lib")
    links = soup1.findAll("a")
    for link in links :
        link_href = link.get('href')
        if "url?q=" in link_href and not "webcache" in link_href:
            link_g = link.get('href').split("?q=")[1].split("&sa=U")[0]
            return link_g
        
@cached(cache=report_cache)
def retinfo(url) -> pd.DataFrame:
    '''Extract tables from a url'''
    new_url = url.replace("https", "http")
    url = requests.get(new_url)
    dfs = pd.read_html(url.text)
    df = dfs[0]
    return df

def comp_name(ticker):
   
    if isinstance(ticker, int):
        ticker = str(ticker)
    if ticker.isnumeric() and len(ticker) == 6:
        with importlib.resources.open_text("mcfinance.data", "dictbse.json") as f:
            dictbse = json.load(f)
        return dictbse[ticker]
    elif ticker.isupper():
        with importlib.resources.open_text("mcfinance.data", "dictnse.json") as f:
            dictnse = json.load(f)
        return dictnse[ticker]        
    else:
        return ticker


import pandas as pd
import mcfinance.retrieval

def prd(url, period, last, trm, a) -> pd.DataFrame:
    '''Retrieve data for multiple years'''
    if period > a or period//5 == trm:
        df = mcfinance.retrieval.retinfo(url)
        df.drop(df.columns[[0,len(df.columns)-1]], inplace = True, axis=1)
        if(period == a+5 and last == 0):
            return df
        else:
            df.drop(columns = df.columns[-last:], inplace = True, axis=1)
        return df

def search_gen(search_term, period) -> pd.DataFrame:
    '''Handles retrieval of data and generation of urls'''
    term = period%5
    last = 5 - period
    #generate datafrane from the search term
    url = mcfinance.retrieval.urlfinder(search_term)
    urls = []
    urls = []
    for i in range(2,5):
        urlt = url + "/" + str(i) + "#" + url.split('/')[6]
        urls.append(urlt)
    #cleaning up the tables
    df1 = mcfinance.retrieval.retinfo(url)
    df1.drop(df1.columns[len(df1.columns)-1], inplace = True, axis = 1)
    if(period == 5 and last == 0):
        fdf = df1
        return fdf
    elif period < 5:
        df1.drop(columns = df1.columns[-last:], inplace = True, axis=1)
    fdf = df1
    i=1
    a = 5
    for url in urls:
        df = prd(url, period, last, i, a)
        fdf = pd.concat([fdf, df], join="inner", ignore_index=True, axis=1)
        i+=1
        a+=5
    
    fdf = fdf.drop(fdf.iloc[:, period+1:],axis = 1)
    
    return fdf

def plo(attribute, search_term, period):
    print("plotting...")
    df = search_gen(search_term, period)
    col_names = df.loc[0, :].values.flatten().tolist()
    X = col_names[1:]
    X.reverse()
    if(df[df.columns[0]] == attribute).any():
        rows = df.loc[df[df.columns[0]] == attribute].squeeze().tolist()
    else:
        raise Exception("attribute does not exist in this file")
        return
    Y = rows[1:]
    Y.reverse()
    Y = [float(i) for i in Y]
    return X,Y


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()


VERSION = "0.1.5"
DESCRIPTION = "Money control financial data extractor-Requires python 3.7+"
LONG_DESCRIPTION = long_description

setup(name="mcfinance",
      version=VERSION,
      author="05kashyap-ragha1992",
      author_email="<aryankashyapnaveen@gmail.com>",
      description=DESCRIPTION,
      long_description_content_type = "text/markdown",
      long_description= LONG_DESCRIPTION,
      packages=['mcfinance'],
      package_dir={'mcfinance': 'src/mcfinance'},
      package_data={'mcfinance': ['data/*.json']},
      install_requires = ['pandas', 'bs4', 'html5lib', 'lxml','matplotlib','requests'],
      keywords=['python','financial','extractor','data'],
      classifiers=["Development Status :: 3 - Alpha",
                   "Intended Audience :: Developers",
                   "Programming Language :: Python :: 3.11",
                   "Operating System :: Microsoft :: Windows",
                   ]
      )